/*
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *          http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.stock.crop;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import nl.cloudfarming.client.crop.model.*;
import nl.cloudfarming.client.model.AgroURI;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.stock.crop.storage.CropDefinitionStorage;
import org.openide.modules.ModuleInstall;
import org.openide.util.Lookup;

/**
 * Will be run when module is loaded. Will load application global
 * cropdefinitionstorage Will register listeners on all cropdefs, genera and
 * varieties in the cropdefstorage and listen/act to changes
 *
 * @author Gerben Feenstra
 */
public class Installer extends ModuleInstall implements PropertyChangeListener {

    /**
     * Will be run when module is loaded. Will load application global
     * cropdefinitionstorage Will register listeners on all cropdefs, genera and
     * varieties in the cropdefstorage
     */
    @Override
    public void restored() {
        registerListeners();
        createDummyValues();
    }

    private void createDummyValues() {
        if (CropDefinitionStorage.getInstance().getCropDefinitions().isEmpty()) {
            DummyCropDefinitionCreator.createDummyCropDefinitions();
        }
    }

    /**
     * Register listeners on cropdefs, genera and varieties
     */
    protected void registerListeners() {
        CropDefinitionStorage storageUnit = CropDefinitionStorage.getInstance();
        storageUnit.addPropertyChangeListener(this);
        for (CropDefinition cropDefinition : storageUnit.getCropDefinitions()) {
            cropDefinition.addPropertyChangeListener(CropDefinition.PROP_NAME, this);
            cropDefinition.addPropertyChangeListener(CropDefinition.PROP_SOURCE, this);
            cropDefinition.addPropertyChangeListener(CropDefinition.PROP_COLOR, this);
            cropDefinition.addPropertyChangeListener(CropDefinition.PROP_CLASSIFICATIONS, this);
            cropDefinition.addPropertyChangeListener(CropDefinition.PROP_VARIETIES, this);

            for (Classification classification : cropDefinition.getClassifications()) {
                classification.addPropertyChangeListener(Classification.PROP_NAME, this);
                classification.addPropertyChangeListener(Classification.PROP_SOURCE, this);
            }
            for (Variety variety : cropDefinition.getVarieties()) {
                variety.addPropertyChangeListener(Variety.PROP_NAME, this);
                variety.addPropertyChangeListener(Variety.PROP_SOURCE, this);
            }
        }
    }

    /**
     * PropertyChange triggered by changes, addition or removal of cropdefs,
     * genera or varieties.
     */
    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        if (evt.getNewValue() == null) { // Remove
            if (evt.getPropertyName().equals(CropDefinitionStorage.PROP_CROP_DEFINITION_ADDREMOVE)) {
                CropDefinition toRemove = (CropDefinition) evt.getOldValue();

                // Remove listeners from genera of this cropdef
                for (Classification classification : toRemove.getClassifications()) {
                    classification.removePropertyChangeListener(this);
                }

                // Remove listeners from varieties of this cropdef
                for (Variety variety : toRemove.getVarieties()) {
                    variety.removePropertyChangeListener(this);
                }
                toRemove.removePropertyChangeListener(this);
            }
            if (evt.getPropertyName().equals(CropDefinition.PROP_CLASSIFICATIONS)) {
                ((Classification) evt.getOldValue()).removePropertyChangeListener(this);
            }
            if (evt.getPropertyName().equals(CropDefinition.PROP_VARIETIES)) {
                ((Variety) evt.getOldValue()).removePropertyChangeListener(this);
            }
        } else { // Add
            if (evt.getPropertyName().equals(CropDefinitionStorage.PROP_CROP_DEFINITION_ADDREMOVE)) {
                ((CropDefinition) evt.getNewValue()).addPropertyChangeListener(this);
            }
            if (evt.getPropertyName().equals(CropDefinition.PROP_CLASSIFICATIONS)) {
                ((Classification) evt.getNewValue()).addPropertyChangeListener(this);
            }
            if (evt.getPropertyName().equals(CropDefinition.PROP_VARIETIES)) {
                ((Variety) evt.getNewValue()).addPropertyChangeListener(this);
            }
        }
    }

    private static class DummyCropDefinitionCreator {

        public static void createDummyCropDefinitions() {
            saveCropDefinition(createDummyCropDefinition("Corn", Color.ORANGE));
            saveCropDefinition(createDummyCropDefinition("Peas", Color.GREEN));
            saveCropDefinition(createDummyCropDefinition("Potatoes", new Color(139, 69, 19), new String[]{"Nicola", "Opperdoeser", "Eigenheimer"}));
            saveCropDefinition(createDummyCropDefinition("Carrots", new Color(255, 125, 0)));
            saveCropDefinition(createDummyCropDefinition("Wheat", Color.YELLOW));
        }

        private static CropDefinition createDummyCropDefinition(String name, Color color) {
            return createDummyCropDefinition(name, color, new String[]{"Variety 1", "Variety 2", "Variety 3"});
        }

        private static CropDefinition createDummyCropDefinition(String name, Color color, String[] varietyNames) {
            final EnterpriseIdProvider EIP = new AgroURI("OTHER/00000000/CTN/1");
            CropDefinition cropDefinition = new CropDefinition(EIP, createDummyCrop(name));
            cropDefinition.setSource("Dummy");
            cropDefinition.setColor(color);
            cropDefinition = addClassifications(cropDefinition);
            cropDefinition = addVarieties(cropDefinition, varietyNames);

            return cropDefinition;
        }

        private static Crop createDummyCrop(String name) {
            return new Crop(new AgroURI("OTHER/00000000/CRP/1"), name);
        }

        private static CropDefinition addClassifications(CropDefinition cropDefinition) {
            final EnterpriseIdProvider EIP = new AgroURI("OTHER/00000000/CLN/1");
            cropDefinition.addClassification(new Classification(EIP, "Class A", "Dummy"));
            cropDefinition.addClassification(new Classification(EIP, "Class B", "Dummy"));

            return cropDefinition;
        }

        private static CropDefinition addVarieties(CropDefinition cropDefinition, String[] varietyNames) {
            final EnterpriseIdProvider EIP = new AgroURI("OTHER/00000000/VTY/1");
            for (int i = 0; i < varietyNames.length; i++) {
                cropDefinition.addVariety(new Variety(EIP, varietyNames[i]));
            }
            return cropDefinition;
        }

        private static void saveCropDefinition(CropDefinition cropDefinition) {
            CropManager cropManager = Lookup.getDefault().lookup(CropManager.class);
            cropManager.save(cropDefinition.getCrop());

            CropDefinitionManager cropDefinitionManager = Lookup.getDefault().lookup(CropDefinitionManager.class);
            cropDefinitionManager.save(cropDefinition);
        }
    }
}
