/*
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *          http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.stock.crop.storage;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.util.Collections;
import java.util.Set;
import nl.cloudfarming.client.crop.model.CropDefinition;
import nl.cloudfarming.client.crop.model.CropDefinitionManager;
import nl.cloudfarming.client.crop.model.CropManager;
import org.openide.util.Lookup;

/**
 * Storage object to store cropdefinitions
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
public class CropDefinitionStorage implements Serializable {

    private static final long serialVersionUID = 1L;
    private transient PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);
    private static CropDefinitionStorage instance;
    public static final String PROP_CROP_DEFINITION_ADDREMOVE = "cropDefinitionAddRemove";

    /**
     * Get a singleton cropstorage instance
     *
     * @return Singleton instance
     */
    public static CropDefinitionStorage getInstance() {
        if (instance == null) {
            instance = new CropDefinitionStorage();
        }
        return instance;
    }

    protected PropertyChangeSupport getPropertyChangeSupport() {
        if (propertyChangeSupport == null) {
            propertyChangeSupport = new PropertyChangeSupport(this);
        }
        return propertyChangeSupport;
    }

    /**
     * Get an unmodifiable list of cropdefinitions in the cropstorage
     *
     * @return The list of cropdefinitions
     */
    public Set<CropDefinition> getCropDefinitions() {
        return Collections.unmodifiableSet(getCropDefinitionManager().findAll());
    }

    /**
     * Remove a cropDefinition from the cropstorage
     *
     * @param cropDefinition Cropdefinition to remove
     */
    public void removeCropDefinition(CropDefinition cropDefinition) {
        getCropDefinitionManager().remove(cropDefinition); 
        getPropertyChangeSupport().firePropertyChange(PROP_CROP_DEFINITION_ADDREMOVE, cropDefinition, null);
    }

    /**
     * Add the specified cropDefinition to the cropStorage and fire
     * propertychange
     *
     * @param cropDefinition
     */
    public void addCropDefinition(CropDefinition cropDefinition) {
        getCropManager().save(cropDefinition.getCrop());
        
        getCropDefinitionManager().save(cropDefinition);
        getPropertyChangeSupport().firePropertyChange(PROP_CROP_DEFINITION_ADDREMOVE, null, cropDefinition);
    }
    
    private CropManager getCropManager() {
        return Lookup.getDefault().lookup(CropManager.class);
    }

    /**
     * Add a propertychangelistener to this instance
     *
     * @param property To listen to
     * @param listener The listener to register
     */
    public void addPropertyChangeListener(String property, PropertyChangeListener listener) {
        getPropertyChangeSupport().addPropertyChangeListener(property, listener);
    }

    /**
     * Add a propertychangelistener to this instance, listen to everything
     *
     * @param listener The listener to register
     */
    public void addPropertyChangeListener(PropertyChangeListener pcl) {
        getPropertyChangeSupport().addPropertyChangeListener(pcl);
    }

    /**
     * Remove a propertychangelistener from this instance
     *
     * @param property The property the registered listener is listening to
     * @param listener The registered listener to remove
     */
    public void removePropertyChangeListener(String property, PropertyChangeListener listener) {
        getPropertyChangeSupport().removePropertyChangeListener(property, listener);
    }

    private CropDefinitionManager getCropDefinitionManager() {
        return Lookup.getDefault().lookup(CropDefinitionManager.class);
    }
    
    public void saveAll() {
        Set<CropDefinition> cropDefinitions = getCropDefinitionManager().findAll();
        for(CropDefinition cropDef : cropDefinitions) {
            getCropManager().save(cropDef.getCrop());
            getCropDefinitionManager().save(cropDef);
        }
    }
}
